import Window from '../../../src/window/Window';
import Document from '../../../src/nodes/document/Document';
import IHTMLOptionElement from '../../../src/nodes/html-option-element/IHTMLOptionElement';
import IHTMLSelectElement from '../../../src/nodes/html-select-element/IHTMLSelectElement';

describe('HTMLOptionElement', () => {
	let window: Window;
	let document: Document;
	let element: IHTMLOptionElement;

	beforeEach(() => {
		window = new Window();
		document = window.document;
		element = <IHTMLOptionElement>document.createElement('option');
	});

	describe('Object.prototype.toString', () => {
		it('Returns `[object HTMLOptionElement]`', () => {
			expect(Object.prototype.toString.call(element)).toBe('[object HTMLOptionElement]');
		});
	});

	describe('get value()', () => {
		it('Returns the attribute "value".', () => {
			element.setAttribute('value', 'VALUE');
			expect(element.value).toBe('VALUE');
		});
	});

	describe('set value()', () => {
		it('Sets the attribute "value".', () => {
			element.value = 'VALUE';
			expect(element.getAttribute('value')).toBe('VALUE');
		});
	});

	describe('get disabled()', () => {
		it('Returns the attribute "disabled".', () => {
			element.setAttribute('disabled', '');
			expect(element.disabled).toBe(true);
		});
	});

	describe('set disabled()', () => {
		it('Sets the attribute "disabled".', () => {
			element.disabled = true;
			expect(element.getAttribute('disabled')).toBe('');
		});
	});

	describe('get selected()', () => {
		it('Returns the selected state of the option.', () => {
			const select = <IHTMLSelectElement>document.createElement('select');
			const option1 = <IHTMLOptionElement>document.createElement('option');
			const option2 = <IHTMLOptionElement>document.createElement('option');

			expect(option1.selected).toBe(false);
			expect(option2.selected).toBe(false);

			select.appendChild(option1);
			select.appendChild(option2);

			expect(option1.selected).toBe(true);
			expect(option2.selected).toBe(false);
			expect(option1.getAttribute('selected')).toBe(null);
			expect(option2.getAttribute('selected')).toBe(null);

			select.options.selectedIndex = 1;

			expect(option1.selected).toBe(false);
			expect(option2.selected).toBe(true);
			expect(option1.getAttribute('selected')).toBe(null);
			expect(option2.getAttribute('selected')).toBe(null);

			select.options.selectedIndex = -1;

			expect(option1.selected).toBe(false);
			expect(option2.selected).toBe(false);
		});
	});

	describe('set selected()', () => {
		it('Sets the selected state of the option.', () => {
			const select = <IHTMLSelectElement>document.createElement('select');
			const option1 = <IHTMLOptionElement>document.createElement('option');
			const option2 = <IHTMLOptionElement>document.createElement('option');

			expect(option1.selected).toBe(false);
			expect(option2.selected).toBe(false);

			option1.selected = true;

			expect(select.selectedIndex).toBe(-1);

			select.appendChild(option1);
			select.appendChild(option2);

			option1.selected = true;

			expect(select.selectedIndex).toBe(0);

			option2.selected = true;

			expect(select.selectedIndex).toBe(1);

			option2.selected = false;

			expect(select.selectedIndex).toBe(0);
		});
	});
});
